// @flow
export const tab: number = 9;
export const enter: number = 13;
export const escape: number = 27;
export const space: number = 32;
export const pageUp: number = 33;
export const pageDown: number = 34;
export const end: number = 35;
export const home: number = 36;
export const arrowLeft: number = 37;
export const arrowUp: number = 38;
export const arrowRight: number = 39;
export const arrowDown: number = 40;
